@@ -1,11 +1,12 @@
 import array
 import asyncio
+import contextlib
 import logging
 import socket
 from collections import deque
 from copy import copy
 from functools import partial
-from typing import Any, Callable, List, Optional, Set, Tuple
+from typing import Any, Callable, Optional
 
 from .. import introspection as intr
 from ..auth import Authenticator, AuthExternal
@@ -20,7 +21,7 @@ from ..constants import (
 from ..errors import AuthError
 from ..message import Message
 from ..message_bus import BaseMessageBus, _block_unexpected_reply
-from ..service import ServiceInterface
+from ..service import ServiceInterface, _Method
 from .message_reader import build_message_reader
 from .proxy_object import ProxyObject
 
@@ -56,7 +57,7 @@ class _MessageWriter:
     def __init__(self, bus: "MessageBus") -> None:
         """A class to handle writing messages to the message bus."""
         self.messages: deque[
-            Tuple[bytearray, Optional[List[int]], Optional[asyncio.Future]]
+            tuple[bytearray, Optional[list[int]], Optional[asyncio.Future]]
         ] = deque()
         self.negotiate_unix_fd = bus._negotiate_unix_fd
         self.bus = bus
@@ -65,7 +66,7 @@ class _MessageWriter:
         self.buf: Optional[memoryview] = None
         self.fd = bus._fd
         self.offset = 0
-        self.unix_fds: Optional[List[int]] = None
+        self.unix_fds: Optional[list[int]] = None
         self.fut: Optional[asyncio.Future] = None
 
     def write_callback(self, remove_writer: bool = True) -> None:
@@ -207,7 +208,7 @@ class MessageBus(BaseMessageBus):
             self._auth = auth
 
         self._disconnect_future = self._loop.create_future()
-        self._pending_futures: Set[asyncio.Future] = set()
+        self._pending_futures: set[asyncio.Future] = set()
 
     async def connect(self) -> "MessageBus":
         """Connect this message bus to the DBus daemon.
@@ -422,7 +423,19 @@ class MessageBus(BaseMessageBus):
         """
         return await self._disconnect_future
 
-    def _make_method_handler(self, interface, method):
+    def _future_exception_no_reply(self, fut: asyncio.Future) -> None:
+        """Log an exception from a future that was not expected."""
+        self._pending_futures.discard(fut)
+        try:
+            fut.result()
+        except asyncio.CancelledError:
+            pass
+        except Exception as e:
+            logging.error("unexpected exception in future", exc_info=e)
+
+    def _make_method_handler(
+        self, interface: "ServiceInterface", method: "_Method"
+    ) -> Callable[[Message, Callable[[Message], None]], None]:
         if not asyncio.iscoroutinefunction(method.fn):
             return super()._make_method_handler(interface, method)
 
@@ -435,7 +448,7 @@ class MessageBus(BaseMessageBus):
         ) -> None:
             """A coroutine method handler."""
             args = msg_body_to_args(msg) if msg.unix_fds else msg.body
-            fut = asyncio.ensure_future(method.fn(interface, *args))
+            fut: asyncio.Future = asyncio.ensure_future(method.fn(interface, *args))
             # Hold a strong reference to the future to ensure
             # it is not garbage collected before it is done.
             self._pending_futures.add(fut)
@@ -443,7 +456,7 @@ class MessageBus(BaseMessageBus):
                 send_reply is _block_unexpected_reply
                 or msg.flags.value & NO_REPLY_EXPECTED_VALUE
             ):
-                fut.add_done_callback(self._pending_futures.discard)
+                fut.add_done_callback(self._future_exception_no_reply)
                 return
 
             # We only create the closure function if we are actually going to reply
@@ -522,6 +535,8 @@ class MessageBus(BaseMessageBus):
         except Exception:
             logging.warning("could not remove message writer", exc_info=True)
 
+        had_handlers = bool(self._method_return_handlers or self._user_message_handlers)
+
         super()._finalize(err)
 
         if self._disconnect_future.done():
@@ -529,6 +544,14 @@ class MessageBus(BaseMessageBus):
 
         if err and not self._user_disconnect:
             _future_set_exception(self._disconnect_future, err)
+            # If this happens during a reply, the message handlers
+            # will have the exception set and wait_for_disconnect will
+            # never be called so we need to manually set the exception
+            # as retrieved to avoid asyncio warnings when the future
+            # is garbage collected.
+            if had_handlers:
+                with contextlib.suppress(Exception):
+                    self._disconnect_future.exception()
         else:
             _future_set_result(self._disconnect_future, None)
 
