@@ -2,7 +2,7 @@ import io
 import json
 import os
 from enum import Enum
-from typing import Any, Dict
+from typing import Any
 
 import pytest
 
@@ -30,7 +30,7 @@ with open(os.path.dirname(__file__) + "/data/get_managed_objects.hex") as fp:
     get_managed_objects_msg = fp.read()
 
 
-def json_to_message(message: Dict[str, Any]) -> Message:
+def json_to_message(message: dict[str, Any]) -> Message:
     copy = dict(message)
     if "message_type" in copy:
         copy["message_type"] = MessageType(copy["message_type"])
@@ -101,8 +101,6 @@ def test_marshalling_with_table():
 
 @pytest.mark.parametrize("unmarshall_table", (table,))
 def test_unmarshalling_with_table(unmarshall_table):
-    from dbus_fast._private import unmarshaller
-
     for item in unmarshall_table:
         stream = io.BytesIO(bytes.fromhex(item["data"]))
         unmarshaller = Unmarshaller(stream)
